/* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
/*
  data_interpretor.c
  heraia - an hexadecimal file editor and analyser based on ghex
 
  (C) Copyright 2005 Olivier Delhomme
  e-mail : heraia@delhomme.org
  URL    : http://heraia.tuxfamily.org
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or  (at your option) 
  any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY;  without even the implied warranty of
  MERCHANTABILITY  or  FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */

#include <gtk/gtk.h>
#include <gtkhex/gtkhex.h>

#include "data_interpretor.h"

/* Begining of data interpretor things  */

/*
    Here we do interpret an unsigned byte in decimal format
*/
static void intrepret_unsigned_byte( GtkWidget *hexwidget, 
									 Data_Window_Struct *DW )
{	
	guint pos;      /* position of the cursor */
	gchar *text;    /* interpreted text to be printed */
	guchar c;       /* the character under the cursor */
	GtkHex *gh;

	gh = GTK_HEX (hexwidget);

	pos = gtk_hex_get_cursor (gh);

	/* test if were are in the file that we are curently editing */
	if((pos >= 0) && (pos < gh->document->file_size))
		{
			c = gtk_hex_get_byte(gh, pos);
			text = (gchar *) g_malloc0 (H_DI_MAX_ENTRY*sizeof(gchar));
			sprintf(text, "%u", c);
			gtk_entry_set_text (GTK_ENTRY(DW->entry_unsigned_byte), text);
		}
	else
		{
			text = (gchar *) g_malloc0 (H_DI_MAX_ENTRY*sizeof(gchar));
			sprintf(text, "%s", "Not a number");
			gtk_entry_set_text (GTK_ENTRY(DW->entry_unsigned_byte), text);
		}
}

/*
    Here we do interpret a signed byte in decimal format
*/
static void intrepret_signed_byte( GtkWidget *hexwidget, 
								   Data_Window_Struct *DW )
{	
	guint pos;      /* position of the cursor */
	gchar *text;    /* interpreted text to be printed */
	gchar c;        /* the character under the cursor */
	GtkHex *gh;

	gh = GTK_HEX (hexwidget);

	pos = gtk_hex_get_cursor (gh);

	/* test if were are in the file that we are curently editing */
	if((pos >= 0) && (pos < gh->document->file_size))
		{
			c = gtk_hex_get_byte(gh, pos);
			text = (gchar *) g_malloc0 (H_DI_MAX_ENTRY*sizeof(gchar));
			sprintf(text, "%d", c);
			gtk_entry_set_text (GTK_ENTRY(DW->entry_signed_byte), text);
		}
	else
		{
			text = (gchar *) g_malloc0 (H_DI_MAX_ENTRY*sizeof(gchar));
			sprintf(text, "%s", "Not a number");
			gtk_entry_set_text (GTK_ENTRY(DW->entry_signed_byte), text);
		}
}
/* End of data interpretor things  */


/* Begining of Widget related stuff  */

static gboolean delete_dt_window_event( GtkWidget *widget, 
						      GdkEvent  *event,
						      gpointer   data )
{
  gtk_widget_hide (widget);
  return TRUE;
}

static void destroy_dt_window( GtkWidget *widget,
					     gpointer   data )
{
  gtk_widget_hide (widget);
}

static void refresh_data_window( GtkWidget *hexwidget, Data_Window_Struct *DW )
{

	gtk_widget_show_all (DW->window);
	intrepret_unsigned_byte(hexwidget, DW);
	intrepret_signed_byte(hexwidget, DW);
}

/* End of Widget related stuff  */


/*
  Initialise the Data_Window_Struct whose aim is to 
  display the data interpreted. Following interpret
  is done automatically :
  - None !
*/
void data_interpret( GtkWidget *hexwidget, Data_Window_Struct *DW )
{

	DW->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	  
	DW->window_vbox = gtk_vbox_new (FALSE, H_DI_V_SPACE);
	gtk_container_add (GTK_CONTAINER(DW->window), DW->window_vbox);
  	
	/* unsigned byte */
	DW->window_hbox_unsigned = gtk_hbox_new (FALSE, H_DI_H_SPACE);
	DW->label_unsigned_byte = gtk_label_new ("unsigned byte (8bits)");
	gtk_container_add (GTK_CONTAINER(DW->window_hbox_unsigned), DW->label_unsigned_byte);
	DW->entry_unsigned_byte = gtk_entry_new_with_max_length (H_DI_MAX_ENTRY);
	gtk_container_add (GTK_CONTAINER(DW->window_hbox_unsigned), DW->entry_unsigned_byte);
	gtk_container_add (GTK_CONTAINER(DW->window_vbox), DW->window_hbox_unsigned);

	/* signed byte */
	DW->window_hbox_signed = gtk_hbox_new (FALSE, H_DI_H_SPACE); 
	DW->label_signed_byte = gtk_label_new ("signed byte (8bits)");
	gtk_container_add (GTK_CONTAINER(DW->window_hbox_signed), DW->label_signed_byte);
	DW->entry_signed_byte = gtk_entry_new_with_max_length (H_DI_MAX_ENTRY);
	gtk_container_add (GTK_CONTAINER(DW->window_hbox_signed), DW->entry_signed_byte);
	gtk_container_add (GTK_CONTAINER(DW->window_vbox), DW->window_hbox_signed);


	g_signal_connect (G_OBJECT (DW->window), "delete_event", 
					  G_CALLBACK (delete_dt_window_event), NULL);
	g_signal_connect (G_OBJECT (DW->window), "destroy", 
					  G_CALLBACK (destroy_dt_window), NULL);

	g_signal_connect (G_OBJECT (hexwidget), "cursor_moved",
					  G_CALLBACK (refresh_data_window), DW);

	/* different init calls */
	intrepret_unsigned_byte(hexwidget, DW);
	intrepret_signed_byte(hexwidget, DW);
	gtk_widget_show_all (DW->window);
}
